/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "torus.h"
#include "../newtEngine/scene.h"

TorusMesh::TorusMesh(float bigR, int nbBigR, float smallR, int nbSmallR, bool inside):	
	Mesh("torus", false, true, false, true), mInside(inside), mBigR(bigR), mSmallR(smallR) {
	
	float meshLevel = scene->getGraphicOptions()->getMeshLevel();
	float value;
		
	value = (float) nbBigR * meshLevel;
	if(value < 4) {
		mNbBigR = 4;
	} else {
		mNbBigR = (int) value;
	}

	value = (float) nbSmallR * meshLevel;
	if(value < 4) {
		mNbSmallR = 4;
	} else {
		mNbSmallR = (int) value;
	}				
}

TorusMesh::~TorusMesh() {
}
		
void TorusMesh::initFirstParams() {
	geometry.nbVertices = (mNbBigR + 1) * ( mNbSmallR + 1);
	
	geometry.nbIndexArrays = mNbSmallR;
	geometry.primType = new GLenum[mNbSmallR];
	for(int i=0;i<mNbSmallR;i++) {
		geometry.primType[i] = GL_QUAD_STRIP;
	}
	
	geometry.nbIndices = new int[mNbSmallR];
	for(int i=0;i<mNbSmallR;i++) {
		geometry.nbIndices[i] = 2 * (mNbBigR+1);
	}	
}
		
void TorusMesh::createVertices(Vector3 *ptr) {
	float u, v;
	
	for(int i=0;i<=mNbSmallR;i++) {
		u = ((float) i/mNbSmallR + 1./2.)*2.*M_PI; 
	
		for(int j=0;j<mNbBigR;j++) {
			v = ((float) j/mNbBigR)*2.*M_PI; 
			*ptr = Vector3((mBigR + mSmallR * cos(u))*sin(v), mSmallR * sin(u), (mBigR + mSmallR * cos(u))*cos(v));
	    	ptr ++;
		}
		*ptr = Vector3(0., mSmallR * sin(u), mBigR + mSmallR * cos(u));
	    ptr ++;
	}	
}

void TorusMesh::createColors(Color *ptr) {
	
}

void TorusMesh::createNormals(Vector3 *ptr) {
	float u, v;
	
	if(mInside) {
		for(int i=0;i<=mNbSmallR;i++) {
			u = ((float) i/mNbSmallR)*2.*M_PI; 
		
			for(int j=0;j<mNbBigR;j++) {
				v = ((float) j/mNbBigR)*2.*M_PI; 
				*ptr = Vector3(cos(u)*sin(v), sin(u), cos(u)*cos(v));
				ptr ++;
			}
			*ptr = Vector3(0., sin(u), cos(u));
			ptr ++;
		}		
	} else {
		for(int i=0;i<=mNbSmallR;i++) {
			u = ((float) i/mNbSmallR)*2.*M_PI; 
		
			for(int j=0;j<mNbBigR;j++) {
				v = ((float) j/mNbBigR)*2.*M_PI; 
				*ptr = Vector3(-cos(u)*sin(v), -sin(u), -cos(u)*cos(v));
				ptr ++;
			}
			*ptr = Vector3(0., -sin(u), -cos(u));
			ptr ++;
		}			
	}
}

void TorusMesh::createTexCoords(GLfloat *ptr) {
	float u, v;
	
	for(int i=0;i<=mNbSmallR;i++) {
		u = (float) i/mNbSmallR; 
	
		for(int j=0;j<mNbBigR;j++) {
			v = (float) j/mNbBigR; 
			*ptr = u;
			ptr ++;
			*ptr = v;
			ptr ++;			
		}
		*ptr = u;
		ptr ++;
		*ptr = 1.;
		ptr ++;
	}		
}

void TorusMesh::createFogCoords(GLfloat *ptr) {
	
}
		
void TorusMesh::createTangents(Vector3 *ptr) {
	float u, v;

	for(int i=0;i<=mNbSmallR;i++) {
		u = ((float) i/mNbSmallR)*2.*M_PI; 
		
		for(int j=0;j<mNbBigR;j++) {
			v = ((float) j/mNbBigR)*2.*M_PI; 
			if(mInside) {
				*ptr = Vector3(-cos(v),0.,sin(v));
			} else {
				*ptr = Vector3(cos(v),0.,-sin(v));
			}
	    	ptr ++;
		}
		if(mInside) {	
			*ptr = Vector3(-1.,0.,0.);
		} else {
			*ptr = Vector3(1.,0.,0.);
		}		
	    ptr ++;		
	}	
}
	
		
void TorusMesh::createIndices(int numArray, GLuint *ptr) {
	if(mInside) {	
		for(int j=0;j<=mNbBigR;j++) {
			*ptr = numArray * (mNbBigR+1) + j;
			ptr ++;
			*ptr = (numArray+1) * (mNbBigR+1) + j;
			ptr ++;
		}

	} else {
		for(int j=0;j<=mNbBigR;j++) {
			*ptr = (numArray+1) * (mNbBigR+1) + j;
			ptr ++;			
			*ptr = numArray * (mNbBigR+1) + j;
			ptr ++;
		}	
	}	
}
	







